# Source:
# https://docs.marimo.io/guides/working_with_data/dataframes.html
# https://github.com/marimo-team/marimo/blob/main/examples/sql/querying_dataframes.py
# https://github.com/crate/cratedb-examples/blob/main/notebook/marimo/basic.py

# /// script
# requires-python = ">=3.10"
# dependencies = [
#     "duckdb",
#     "marimo",
#     "pandas==2.2.3",
#     "sqlalchemy-cratedb",
#     "sqlalchemy==2.0.36",
# ]
# ///

import marimo

__generated_with = "0.10.2"
app = marimo.App(width="medium")


@app.cell
def _():
    import marimo as mo
    return (mo,)


@app.cell
def _():
    cratedb_sqlalchemy_url = "crate://"
    return (cratedb_sqlalchemy_url,)


@app.cell(hide_code=True)
def _(mo, cratedb_sqlalchemy_url):
    mo.md(
        """
        # Connect to CrateDB or CrateDB Cloud

        This notebook shows how to use Marimo and CrateDB, with Python dataframes and SQL.
        First, let's create a dataframe called `df`, populated with content from CrateDB's
        built-in `sys.summits` table.
        """
    )


@app.cell
def _(mo):
    settings = mo.vstack(
        [
            mo.md("Edit User"),
            first := mo.ui.text(label="First Name"),
            last := mo.ui.text(label="Last Name"),
        ]
    )

    organization = mo.vstack(
        [
            mo.md("Edit Organization"),
            org := mo.ui.text(label="Organization Name", value="..."),
            employees := mo.ui.number(
                label="Number of Employees", start=0, stop=1000
            ),
        ]
    )

    mo.ui.tabs(
        {
            "🧙‍♀ User": settings,
            "🏢 Organization": organization,
        }
    )
    return employees, first, last, org, organization, settings


@app.cell
def _(employees, first, last, mo, org):
    mo.md(
        f"""
        Welcome **{first.value} {last.value}** to **{org.value}**! You are 
        employee no. **{employees.value + 1}**.

        #{"🎉" * (min(employees.value + 1, 1000))} 
        """
    ) if all([first.value, last.value, org.value]) else mo.md(
        "Type a first and last name!"
    )
    return


@app.cell(hide_code=True)
def _(mo, cratedb_sqlalchemy_url):
    mo.md(
        f"""
        Connecting to CrateDB at **`{cratedb_sqlalchemy_url}`**.
        """
    )


@app.cell
def _(mo, cratedb_sqlalchemy_url):
    import pandas as pd
    import sqlalchemy as sa

    engine = sa.create_engine(cratedb_sqlalchemy_url)
    df = pd.read_sql(sql="SELECT * FROM sys.summits ORDER BY height DESC LIMIT 5", con=engine)

    print(df)
    df
    return df, engine, pd, sa


if __name__ == "__main__":
    app.run()



# Source:
# https://docs.marimo.io/guides/working_with_data/dataframes.html
# https://github.com/marimo-team/marimo/blob/main/examples/sql/querying_dataframes.py
# https://github.com/crate/cratedb-toolkit/blob/3abe9830528fd708ef39394e578fe35918e4d651/cratedb_toolkit/cfr/marimo.py

# /// script
# requires-python = ">=3.10"
# dependencies = [
#     "duckdb",
#     "marimo",
#     "pandas==2.2.3",
#     "sqlalchemy-cratedb",
#     "sqlalchemy==2.0.36",
#     "python-dotenv==1.0.1",
# ]
# ///

import marimo

__generated_with = "0.10.13"
app = marimo.App(width="full", app_title="CrateDB Jobs Analysis")


@app.cell
def _():
    import marimo as mo
    return (mo,)


@app.cell(hide_code=True)
def _():
    import sqlalchemy as sa

    engine = sa.create_engine("crate://crate@localhost:4200/")
    return engine


@app.cell(hide_code=True)
def _(mo):
    mo.md(r"""# sys.jobs_log""")
    return


@app.cell(hide_code=True)
def _(mo):
    mo.md(r"""### Explore""")
    return


@app.cell
def _(mo):
    limitjl=mo.ui.slider(1,30000, step=20, value=100)
    limitjl
    return (limitjl,)


@app.cell(hide_code=True)
def _(engine, limitjl, pd):
    import ast  # Safely parse string representations of dictionaries
    import pandas as pd

    #df2 = pd.read_sql(sql="SELECT stmt, started, ended, username FROM sys.jobs_log limit 10", con=engine)
    #df2 = pd.read_sql(sql="SELECT * FROM sys.jobs_log limit {$limit.value} order by ended", con=engine)
    df2 = pd.read_sql(f"SELECT * FROM sys.jobs_log ORDER BY ended LIMIT {limitjl.value}", con=engine)

    # Convert epoch milliseconds to datetime
    df2['started'] = pd.to_datetime(df2['started'], unit='ms')
    df2['ended'] = pd.to_datetime(df2['ended'], unit='ms')
    df2['duration'] = (df2['ended'] - df2['started']).dt.total_seconds() * 1000


    # Example: Assuming the column is named 'metadata'
    df2['node'] = df2['node'].apply(lambda x: ast.literal_eval(x)['name'] if isinstance(x, str) else x['name'])

    #mo.ui.dataframe(df2)
    # Aggregating the 'duration' column
    #aggregated = df2['duration'].agg(['min', 'max', 'mean']).reset_index()

    # Rename the columns for clarity
    #aggregated.columns = ['aggregation', 'duration_min', 'duration_max', 'duration_mean']

    # Add aggregated values as new columns to df2
    #df2 = df2.join(aggregated[['duration_min', 'duration_max', 'duration_mean']].iloc[0], rsuffix='_agg')

    df2_next = df2
    df2_next = df2_next[["stmt", "started", "ended", "duration", "username", "node"]]
    df2_next
    return ast, df2, df2_next


@app.cell(hide_code=True)
def _(mo):
    mo.md(r"""### Slow Query Log""")
    return


@app.cell
def _(df2_next, mo):
    max_value = len(df2_next)
    slowq=mo.ui.slider(1,max_value, step=20, value=10)
    slowq
    return max_value, slowq


@app.cell
def _(df2_next, slowq):
    # Sort by 'duration' in descending order and get the top 10 slowest queries
    #slowest_queries = df2_next.sort_values(by='duration', ascending=False).head(10)
    # Sort by 'duration' in descending order and get the top 10 slowest queries
    slowq_value = int(slowq.value)  # Convert the slider value to an integer

    # Sort the DataFrame by 'duration' and get the top slowq_value rows
    slowest_queries = df2_next.sort_values(by='duration', ascending=False).head(slowq_value)
    #slowest_queries = df2_next.sort_values(by='duration', ascending=False).head(slowq.value)


    # Display the result
    print(slowest_queries[['stmt', 'duration']])

    # Display the result
    slowest_queries
    return slowest_queries, slowq_value


if __name__ == "__main__":
    app.run()



# Source:
# https://docs.marimo.io/guides/working_with_data/dataframes.html
# https://github.com/marimo-team/marimo/blob/main/examples/sql/querying_dataframes.py
# https://github.com/crate/cratedb-examples/blob/main/notebook/marimo/basic.py

# /// script
# requires-python = ">=3.10"
# dependencies = [
#     "duckdb",
#     "marimo",
#     "pandas==2.2.3",
#     "sqlalchemy-cratedb",
#     "sqlalchemy==2.0.36",
# ]
# ///

import marimo

__generated_with = "0.10.2"
app = marimo.App(width="medium")


@app.cell
def _():
    import marimo as mo
    return (mo,)


@app.cell
def _():
    cratedb_sqlalchemy_url = "crate://cratedb:4200"
    return (cratedb_sqlalchemy_url,)


@app.cell(hide_code=True)
def _(mo):
    mo.md(
        """
        # Querying CrateDB

        This notebook shows how to use Marimo and CrateDB, with Python dataframes and SQL.
        First, let's create a dataframe called `df`, populated with content from CrateDB's
        built-in `sys.summits` table.
        """
    )

@app.cell(hide_code=True)
def _(mo, cratedb_sqlalchemy_url):
    mo.md(
        f"""
        Connecting to CrateDB at **`{cratedb_sqlalchemy_url}`**.
        """
    )


@app.cell
def _(mo, cratedb_sqlalchemy_url):
    import pandas as pd
    import sqlalchemy as sa

    engine = sa.create_engine(cratedb_sqlalchemy_url)
    df = pd.read_sql(sql="SELECT * FROM sys.summits ORDER BY height DESC", con=engine)

    print(df)
    df
    return df, engine, pd, sa


@app.cell(hide_code=True)
def _(mo):
    mo.md(
        f"""
        Next, we **create a SQL cell**.
        In the SQL cell, you can query dataframes in your notebook as if they were tables — just reference them by name.
        For querying a data frame using SQL, Marimo uses DuckDB.
        """
    )
    return


@app.cell(hide_code=False)
def _(mo):
    result = mo.sql(
        f"""
        SELECT * FROM df WHERE region LIKE 'Bernina%' ORDER BY height DESC
        """,
        output=False,
    )
    return (result,)


@app.cell(hide_code=True)
def _(mo):
    mo.md(
        r"""
        The query output is returned to Python as a dataframe (Polars if you have it installed, Pandas otherwise).
        """
    )
    return


@app.cell
def _(result):
    print(result)
    result
    return


if __name__ == "__main__":
    app.run()



#!/usr/bin/env python
"""
Marimo Suraya MK.
"""

# ruff: noqa: S603,S607
import importlib.resources
import logging
import os
import shlex
import subprocess
from textwrap import dedent

import click
import hishel
from pueblo import setup_logging


logger = logging.getLogger(__name__)


# HTTP client, with 1 hour of caching.
storage = hishel.FileStorage(ttl=3600)
http = hishel.CacheClient(storage=storage)

OCI_NAME_DEFAULT = "marimo-suraya:dev"


@click.group()
def cli():
    """
    Marimo Suraya Builder.
    """
    logger.info("Starting Marimo Suraya MK")


@cli.command()
@click.argument("image", type=str, default=OCI_NAME_DEFAULT)
def build(image: str):
    """
    Build OCI image.
    """
    os.environ["BUILDKIT_PROGRESS"] = "plain"
    os.environ["DOCKER_BUILDKIT"] = "1"

    folder = importlib.resources.files(__package__)
    subprocess.check_call(
        [
            "docker",
            "build",
            "--file",
            folder / "Dockerfile",
            "--build-context",
            f"module={folder}",
            "--tag",
            image,
            ".",
        ]
    )


@cli.command()
@click.argument("image", type=str, default=OCI_NAME_DEFAULT)
def run(image: str):
    """
    Run Marimo Suraya using OCI image.
    """
    command = dedent(f"""
    docker run --rm -it --name=marimo-suraya \
    --publish=4211:4211 \
    {image}
    """).replace("\n", "")
    subprocess.check_call(shlex.split(command))



if __name__ == "__main__":
    setup_logging()
    cli()



#!/usr/bin/env python
"""
Grafana Suraya MK.
"""

# ruff: noqa: S603,S607
import importlib.resources
import json
import logging
import os
import shlex
import subprocess
import sys
import typing as t
import urllib.parse
from pathlib import Path
from textwrap import dedent

import attrs
import click
import hishel
import httpx
from munch import Munch
from pueblo import setup_logging
from pypdl import pypdl


logger = logging.getLogger(__name__)


# HTTP client, with 1 hour of caching.
storage = hishel.FileStorage(ttl=3600)
http = hishel.CacheClient(storage=storage)

OCI_NAME_DEFAULT = "grafana-suraya:dev"


@attrs.define()
class Plugin:
    """
    Manage minimal plugin information.
    """

    slug: str
    version: str


@attrs.define()
class GrafanaPluginInfo(Plugin):
    """
    Manage extended plugin information.
    """

    homepage_url: str
    repository_url: str
    package_url: str


class GrafanaPluginCatalog:
    """
    Manage the Grafana plugin catalog.
    """

    URL = "https://grafana.com/api/plugins"

    def __init__(self):
        self.data = Munch.fromDict(http.get(self.URL).json())

    def items(self) -> t.Iterator[Munch]:
        for item in self.data["items"]:
            yield item

    def i2p(self, item: Munch):
        try:
            return self.get_package_info(item)
        except ValueError as ex:
            logger.warning(f"Skipping {item.slug}: {ex}")

    def find_plugin(self, slug: str):
        # TODO: Optimize access by indexing by slug.
        for item in self.items():
            if item.slug == slug:
                return self.i2p(item)
        raise ValueError(f"Plugin {slug} not found")

    def get_plugins_by_prefix(self, prefix: str):
        # TODO: Optimize access by indexing by slug.
        for item in self.items():
            if item.slug.startswith(prefix):
                if value := self.i2p(item):
                    yield value

    def get_package_info(self, item: Munch):
        """
        Get plugin and package information.
        """
        # TODO: Select specific platform based on user choice or parent platform.
        candidates = ["linux-amd64", "any"]
        for candidate in candidates:
            if candidate in item.packages:
                return GrafanaPluginInfo(
                    slug=item.slug,
                    version=item.version,
                    homepage_url=f"https://grafana.com/grafana{item.links[0].href}",
                    repository_url=self.get_repository_url(item.url, item.slug),
                    package_url=f"https://grafana.com{item.packages[candidate].downloadUrl}",
                )
        raise ValueError(f"Package not found or unknown package type: {item.slug}")

    def get_repository_url(self, url: str, slug: str) -> t.Union[str, None]:
        if url == "https://github.com/grafana/plugins-private":
            return None
        if url:
            return url
        if "volkovlabs" in slug:
            return f"https://github.com/VolkovLabs/{slug}"
        return None


@attrs.define()
class PluginList:
    """
    Manage a list of plugin items.
    """

    tpl = "https://grafana.com/api/plugins/{name}/versions/{version}/download?os={os}&arch={arch}"

    items: t.List[Plugin] = attrs.field(default=[])
    _catalog: "GrafanaPluginCatalog" = attrs.field(factory=GrafanaPluginCatalog)

    @property
    def package_urls(self) -> t.List[str]:
        urls = []
        # TODO: Limit number by `[:3]` here.
        for item in self.items:
            if item.slug and item.version:
                url = self.tpl.format(
                    name=item.slug, version=item.version, os="linux", arch="amd64"
                )
            elif item.slug:
                info = self._catalog.find_plugin(slug=item.slug)
                if info is None:
                    logger.error(
                        f"Plugin does not exist in Grafana "
                        f"plugin catalog, skipping: {item.slug}"
                    )
                    continue
                url = info.package_url
            else:
                raise KeyError(f"Plugin not found: {item.slug}")
            urls.append(url)
        return urls

    def add_manifest(self, path: Path):
        if path.suffix == ".json":
            with open(path) as f:
                data = Munch.fromJSON(f.read())
            if "plugins" in data:
                for item in data["plugins"]:
                    self.items.append(Plugin(slug=item["slug"], version=item["version"]))
            else:
                raise NotImplementedError("Manifest file format not supported")

        elif path.suffix == ".toml":
            raise NotImplementedError(
                "Reading plugin manifests from TOML not implemented yet"
            )

        else:
            raise click.FileError(str(path), f"Unsupported file extension: {path.suffix}")
        return self

    def add_package(self, slug: str = None, version: str = None, prefix: str = None):
        if slug and version:
            self.items.append(Plugin(slug=slug, version=version))
        elif prefix:
            for plugin in self._catalog.get_plugins_by_prefix(prefix):
                self.items.append(Plugin(slug=plugin.slug, version=plugin.version))
        else:
            raise KeyError(
                f"Plugin not found or incomplete information: "
                f"slug={slug}, version={version}, prefix={prefix}"
            )
        return self

    def to_manifest(self):
        data = {
            "comment": "Plugins for Amazon Managed Grafana (AMG)",
            "plugins": [
                {"slug": plugin.slug, "version": plugin.version} for plugin in self.items
            ],
        }
        return json.dumps(data, indent=2)


def get_plugins_standard(path: Path) -> PluginList:
    """
    Get standard set of plugins, mostly AMG plus Volkov Labs.
    """
    plugins = PluginList()
    plugins.add_manifest(path).add_package(prefix="volkovlabs-")
    return plugins


@click.group()
def cli():
    """
    Grafana Suraya Builder.
    """
    logger.info("Starting Grafana Suraya MK")


@cli.command()
@click.argument("manifest", type=click.Path(exists=True, path_type=Path))
def update_manifest(manifest: Path):
    """
    Update the plugin manifest to use the most recent version of each plugin.
    """
    logger.info(f"BOM manifest path: {manifest}")
    plugins = get_plugins_standard(manifest)
    for plugin in plugins.items:
        plugin.version = plugins._catalog.find_plugin(plugin.slug).version
    print(plugins.to_manifest())  # noqa: T201


@cli.command()
@click.argument("path", type=click.Path(exists=True, path_type=Path))
def plugin_urls(path: Path):
    """
    Convert plugin manifests to list of URLs.
    """
    logger.info(f"Using manifest path: {path}")
    plugins = get_plugins_standard(path)
    print("\n".join(plugins.package_urls), file=sys.stdout)  # noqa: T201


@cli.command()
@click.argument("manifest", type=click.Path(exists=True, path_type=Path))
@click.argument("target", type=click.Path(exists=True, path_type=Path))
def plugins_download(manifest: Path, target: Path):
    """
    Download plugins by list of URLs.
    """
    logger.info(f"BOM manifest path: {manifest}")
    plugins = get_plugins_standard(manifest)
    acquire(plugins.package_urls, target)


def acquire(urls: t.List[str], target: Path):
    """
    Download and extract list of plugins from a list of URLs into a target folder.
    """

    def get_filename(url: str):
        """
        Only because pypdl does not respect the *redirected* download filename.
        """
        response = http.get(url)
        try:
            response.raise_for_status()
        except httpx.HTTPStatusError as ex:
            if not str(ex).startswith("Redirect response"):
                logger.error(f"Request failed: {ex}")
                # TODO: raise or continue?
                raise
        return Path(urllib.parse.urlsplit(http.get(url).headers["location"]).path).name

    # Build task list for pypdl downloader.
    tasks = []
    for url in urls:
        filename = get_filename(url)
        task = {"url": url, "file_path": str(target / filename)}
        tasks.append(task)

    # Invoke pypdl downloader.
    dl = pypdl.Pypdl(max_concurrent=3, allow_reuse=True)
    results = dl.start(
        tasks=tasks,
        multisegment=False,
        overwrite=False,
        display=True,
        block=True,
        retries=3,
        clear_terminal=False,
    )

    # Display download results.
    for result in results:
        if isinstance(result, tuple):
            url, result = result
            logger.info(f"Downloaded {url}")
        logger.info(f"Download succeeded: {result.path}")

    dl.stop()
    dl.shutdown()


@cli.command()
@click.argument("path", type=click.Path(exists=True, path_type=Path))
@click.pass_context
def plugins_install(ctx, path: Path):
    """
    Install list of plugins using Grafana CLI.
    """
    logger.info(f"Using download BOM path: {path}")
    plugins = get_plugins_standard(path)
    for plugin in plugins.items:
        ctx.invoke(plugin_install, slug=plugin.slug, version=plugin.version)


@cli.command()
@click.argument("slug", type=str, required=True)
@click.argument("version", type=str, required=False)
def plugin_install(slug: str, version: str):
    """
    Install plugin using Grafana CLI.
    """
    logger.info(f"Installing plugin: {slug}/{version}")
    command = f"grafana cli plugins install {slug} {version}"
    subprocess.check_call(shlex.split(command))


@cli.command()
@click.argument("image", type=str, default=OCI_NAME_DEFAULT)
def build(image: str):
    """
    Build OCI image.
    """
    os.environ["BUILDKIT_PROGRESS"] = "plain"
    os.environ["DOCKER_BUILDKIT"] = "1"

    folder = importlib.resources.files(__package__)
    subprocess.check_call(
        [
            "docker",
            "build",
            "--file",
            folder / "Dockerfile",
            "--build-context",
            f"module={folder}",
            "--tag",
            image,
            ".",
        ]
    )


@cli.command()
@click.argument("image", type=str, default=OCI_NAME_DEFAULT)
@click.argument("admin-password", type=str, default="admin")
def run(image: str, admin_password: str):
    """
    Run Grafana using OCI image.
    """
    command = dedent(f"""
    docker run --rm -it --name=grafana-suraya \
    --publish=3000:3000 \
    --env='GF_SECURITY_ADMIN_PASSWORD={admin_password}' \
    {image}
    """).replace("\n", "")
    subprocess.check_call(shlex.split(command))


if __name__ == "__main__":
    setup_logging()
    cli()



# Boot Suraya from Compose recipe.
# https://gabrieldemarmiesse.github.io/python-on-whales/sub-commands/compose/
import logging
from importlib import resources
from importlib.resources import files
from pathlib import Path

import click
from pueblo import setup_logging
from python_on_whales import DockerClient

logger = logging.getLogger(__name__)


class Suraya:
    """
    """
    def __init__(self):
        with resources.as_file(resources.files(__package__) / "compose.yaml") as compose_file:
            self.docker = DockerClient(compose_files=[compose_file])
    def boot(self):
        self.docker.compose.up()


@click.group()
def cli():
    """
    Suraya CLI.
    """
    logger.info("Starting Suraya")


@cli.command()
def boot():
    """
    Start Suraya.
    """
    Suraya().boot()


if __name__ == "__main__":
    setup_logging()
    cli()



# Source: https://github.com/marimo-team/marimo/tree/main/examples/frameworks/fastapi-github
import tempfile
from importlib import resources

from fastapi import FastAPI, Request
from fastapi.templating import Jinja2Templates
import marimo
import os
import logging
import requests
from pathlib import Path

# Set up logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# Constants
templates_dir = os.path.dirname(__file__)

# Set up templates
templates = Jinja2Templates(directory=templates_dir)


def download_github_files(repo: str, path: str = "") -> list[tuple[str, str]]:
    """Download files from GitHub repo, returns list of (file_path, content)"""
    api_url = f"https://api.github.com/repos/{repo}/contents/{path}"
    response = requests.get(api_url)
    response.raise_for_status()

    files: list[tuple[str, str]] = []
    for item in response.json():
        print(item)
        if item["type"] == "file" and item["name"].endswith(".py"):
            content_response = requests.get(item["download_url"])
            files.append(
                (os.path.join(path, item["name"]), content_response.text)
            )
        elif item["type"] == "dir":
            files.extend(
                download_github_files(repo, os.path.join(path, item["name"]))
            )
    return files


tmp_dir = tempfile.TemporaryDirectory()


def setup_apps():
    """Download and setup marimo apps from GitHub"""
    #GITHUB_REPO = os.environ.get("GITHUB_REPO", "marimo-team/marimo")
    #ROOT_DIR = os.environ.get("ROOT_DIR", "examples/ui")
    #files = download_github_files(GITHUB_REPO, ROOT_DIR)
    server = marimo.create_asgi_app()
    app_names: list[str] = []

    """
    for file_path, content in files:
        app_name = Path(file_path).stem
        local_path = Path(tmp_dir.name) / file_path

        # Create directories if they don't exist
        local_path.parent.mkdir(parents=True, exist_ok=True)

        # Write file content
        local_path.write_text(content)

        # Add to marimo server
        server = server.with_app(path=f"/{app_name}", root=str(local_path))
        app_names.append(app_name)
        logger.info(f"Added app: {app_name} from {file_path}")
    """

    marimo_file = Path(__file__).parent / "app" / "basic.py"
    #with resources.as_file(resources.files("suraya.marimo.app") / "basic.py") as marimo_file:
    #with resources.(resources.files("suraya.marimo.app") / "basic.py") as marimo_file:
    for app_file in sorted(Path(resources.files("suraya.marimo.app")).glob("*.py")):
        content = app_file.read_text()
        if "import marimo" in content:
            app_name = app_file.stem
            server = server.with_app(path=f"/{app_name}", root=str(app_file))
            app_names.append(app_name)

    return server, app_names


# Create a FastAPI app
app = FastAPI(root_path="/m")

# Setup marimo apps
server, app_names = setup_apps()


@app.get("/")
async def home(request: Request):
    return templates.TemplateResponse(
        "index.html", {"request": request, "app_names": app_names}
    )


@app.get("/ping")
async def root():
    return {"message": "pong"}


# Mount the marimo server
app.mount("/", server.build())


# Run the server
if __name__ == "__main__":
    import uvicorn

    uvicorn.run(app, host="0.0.0.0", port=4211, log_level="info")
